//------------------------------------------------------------------------------
// Company      : Fidus Systems
// Date         : 07/25/2012
// Designer     : Javier Rodriguez Molina
// Contact      : j.rodriguez.molina@fidus.com
// Design Name  : DC1564A 14 bit ADC
// File Name    : linear_1564a.v
// Hardware     : Stratix IV development board DK-DEV-4SGX230N
// Description  :
// This design captures incoming data from the Linear Technology DC1654A-G board.
// The dual channel 14 bit 310Msps DDR LVDS ADC feeds data to the FPGA. The FPGA
// captures the 7 bit bus (configurable to 8) and deserializes the stream by a
// factor of 4 producing 2 samples (even and odd) at half the speed of the
// incoming data (in this case 155MHz). The captured data from either the odd
// or even sample is outputed to the HSMC connector on the Stratix development
// board for testing and it is also captured by Signal Tap. Refer to the
// DC1564A Design Document for more information
//------------------------------------------------------------------------------

module linear_1564a
(
  //Input Clock
  input                 CLK_IN            ,
  //Input Data
  input      [7:0]      DATA_A_IN         ,
  input      [7:0]      DATA_B_IN         ,
  //User
  input                 CH_SEL            ,
  //SPI loopback
  output                SCK               ,
  output                SDI               ,
  input                 SDO               ,
  output                CSn               ,
  input                 CSn_in            ,
  input                 SCK_in            ,
  input                 SDI_in            ,
  output                SDO_out           ,
  //Output lock
  output                CLK_OUT           ,
  //Ouput data
  output    [15:0]      DATA_OUT
);

////////////////////////////////////////////////////////////////////////////////
//******************************************************************************
//******************************************************************************
//******************************************************************************
//******************************************************************************
////////////////////////////////////////////////////////////////////////////////

// Signal Declaration
wire [31:0]       data_samples_chA;
wire [31:0]       data_samples_chB;
wire [15:0]       data_align_chA;
wire [15:0]       data_align_chB;
wire [15:0]       data_out_chA;
wire [15:0]       data_out_chB;
wire              clk_620mhz;
wire              clk_en;
wire              pll_locked;
wire              clk_155mhz;

//SPI assigment. Modify based on your SPI interface needs
//Refer to the DC1564A Design Document for more information
assign    SCK      = SCK_in;
assign    SDI      = SDI_in;
assign    CSn      = CSn_in;
assign    SDO_out  = SDO;

//Output Data Assigment -- User Dip Switch SW3.1 -- OFF => chA ON => chB
assign DATA_OUT = (CH_SEL)? data_out_chA : data_out_chB;

////////////////////////////////////////////////////////////////////////////////
//Choose sample to output.
//Uncomment EITHER the even or odd sample to output from channel A and B.
//With a deserialization factor of 4 we obtain two 16 bit samples @ 155MHz
//(the 2 LSB are for future used and left are connected to ground) but only
//one of the samples is transmitted. This creates an effective down sample of 2.
//The MSB its inverted to convert from the data format of the DC1564A-G to the 
//data format expected by the DC890B. (Unsigned decimal -> Signed decimal)
//The capture data from the ADC is inverted before entering the FPGA and it is 
//inverted when exciting the FPGA by the adapter board. Keep in mind the data 
//inside the FPGA it is inverted from its original value.
////////////////////////////////////////////////////////////////////////////////
//Even Sample Channel A
assign data_align_chA = {~data_samples_chA[30],data_samples_chA[31],
                    data_samples_chA[26],data_samples_chA[27],
                    data_samples_chA[22],data_samples_chA[23],
                    data_samples_chA[18],data_samples_chA[19],
                    data_samples_chA[14],data_samples_chA[15],
                    data_samples_chA[10],data_samples_chA[11],
                    data_samples_chA[6],data_samples_chA[7],
                    1'b0,1'b0};                                 //Delete for 16 bit ADC
                    //data_samples_chA[2],data_samples_chA[3]}; //Uncomment for 16 bit ADC
//Odd Sample Channel A
//assign data_align_chA = {~data_samples_chA[28],data_samples_chA[29],
//                    data_samples_chA[24],data_samples_chA[27],
//                    data_samples_chA[20],data_samples_chA[23],
//                    data_samples_chA[16],data_samples_chA[19],
//                    data_samples_chA[12],data_samples_chA[15],
//                    data_samples_chA[8],data_samples_chA[11],
//                    data_samples_chA[4],data_samples_chA[7],
//                    1'b0,1'b0};                                 //Delete for 16 bit ADC
//                    //data_samples_chA[0],data_samples_chA[1]}; //Uncomment for 16 bit ADC

//Even Sample Channel B
assign data_align_chB = {~data_samples_chB[30],data_samples_chB[31],
                    data_samples_chB[26],data_samples_chB[27],
                    data_samples_chB[22],data_samples_chB[23],
                    data_samples_chB[18],data_samples_chB[19],
                    data_samples_chB[14],data_samples_chB[15],
                    data_samples_chB[10],data_samples_chB[11],
                    data_samples_chB[6],data_samples_chB[7],
                    1'b0,1'b0};                                 //Delete for 16 bit ADC
                    //data_samples_chB[2],data_samples_chB[3]}; //Uncomment for 16 bit ADC
//Odd Sample Channel B
//assign data_align_chA = {~data_samples_chB[28],data_samples_chB[29],
//                    data_samples_chB[24],data_samples_chB[27],
//                    data_samples_chB[20],data_samples_chB[23],
//                    data_samples_chB[16],data_samples_chB[19],
//                    data_samples_chB[12],data_samples_chB[15],
//                    data_samples_chB[8],data_samples_chB[11],
//                    data_samples_chB[4],data_samples_chB[7],
//                    1'b0,1'b0};                                 //Delete for 16 bit ADC
//                    //data_samples_chB[0],data_samples_chB[1]}; //Uncomment for 16 bit ADC
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Instantiate pll
ext_pll ext_pll_inst (
  .areset   (                       ),
  .inclk0   ( CLK_IN                ),  // Input clock    -  310MHz
  .c0       ( clk_620mhz            ),  // Serial clock   -  620MHz
  .c1       ( clk_en                ),  // Enable clock   -  155Mhz
  .c2       ( clk_155mhz            ),  // Parallel clock -  155MHz
  .locked   ( pll_locked            )   //
  );
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
//The 180 degrees phase shift introduced into the input clock
//by the adapter board DC1933A is removed by triggering the
//LVDS receiver on the FALLING EDGE of the clock instead of
//the rising edge.
//This fix applies only in the case of using the DC1933A 
//adapter board and needs to be modified if not.
////////////////////////////////////////////////////////////////////////////////
//LVDS Receiver Channel A
lvds_rx_ext_pll lvds_receiver_chA (
  .rx_enable             ( clk_en                     ),
  .rx_in                 ( DATA_A_IN                  ),
  .rx_inclock            ( clk_620mhz                 ),
  .rx_out                ( data_samples_chA           )
  );

//LVDS Receiver Channel B
lvds_rx_ext_pll lvds_receiver_chB (
  .rx_enable             ( clk_en                     ),
  .rx_in                 ( DATA_B_IN                  ),
  .rx_inclock            ( clk_620mhz                 ),
  .rx_out                ( data_samples_chB           )
  );
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
//The 180 degrees phase shift introduced into the output clock
//by the adapter board DC1933A is compensated by introducing an 
//an output delay to the Data_out paths with respect to the 
//the output clock.
//This fix applies only in the case of using the DC1933A 
//adapter board and needs to be modified if not.
////////////////////////////////////////////////////////////////////////////////
// LVDS Transmitter: Output Clock
lvds_tx_pass_thru clk_out_lvds (
  .tx_in          ( clk_155mhz        ),
  .tx_out         ( CLK_OUT           )
  );
////////////////////////////////////////////////////////////////////////////////
    
////////////////////////////////////////////////////////////////////////////////
// Deserialized data coming out from the LVDS receiver needs to be registered
genvar i;
generate
  for(i = 0; i < 16; i = i + 1) begin: GEN_LOOP
  DFF data_chA_reg (
    .d              (data_align_chA[i]              ),
    .clk            (clk_155mhz                     ),
    .clrn           (1'b1                           ),
    .prn            (1'b1                           ),
    .q              (data_out_chA[i]                )
    );

  DFF data_chB_reg (
    .d              (data_align_chB[i]              ),
    .clk            (clk_155mhz                     ),
    .clrn           (1'b1                           ),
    .prn            (1'b1                           ),
    .q              (data_out_chB[i]                )
    );
  end
endgenerate
////////////////////////////////////////////////////////////////////////////////

endmodule
